<?php

namespace App\Controller;

use App\Entity\Counter;
use App\Form\DataTransformer\MinutesToPeriodTransformer;
use App\Repository\CounterRepository;
use App\Service\CounterManager;
use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Attribute\Route;
use Symfony\Component\Workflow\Exception\LogicException;
use Symfony\Component\Workflow\WorkflowInterface;

#[Route('/counter')]
class CounterController extends AbstractController
{

    public function __construct(
        protected CounterManager $counterManager,
        protected WorkflowInterface $countdownStateMachine,
        protected CounterRepository $counterRepository
    )
    {
        
    }

    #[Route('/', name: 'app_counter', methods: ['GET'])]
    public function index(CounterRepository $counterRepository): JsonResponse
    {
        $counters = $counterRepository->findAll();
        return $this->json($counters, 200);
    }

    #[Route('/', name: 'app_counter_post', methods: ['POST'])]
    public function postCounter(
        Request $request,
        MinutesToPeriodTransformer $minutesToPeriodTransformer
    ): JsonResponse
    {
        $ttl = $request->toArray()['ttl'] ?? null;
        $name = $request->toArray()['name'] ?? null;

        $counter = new Counter();
        $counter
            ->setTimeTolive($minutesToPeriodTransformer->reverseTransform($ttl))
            ->setName($name);

        $this->countdownStateMachine->getMarking($counter);

        return $this->json($counter, 200);
    }

    #[Route('/start/{id}', name: 'app_counter_start')]
    public function startCounter(Counter $counter): JsonResponse
    {
        try {
            $this->countdownStateMachine->apply($counter, Counter::TRANSITION_TO_STARTED);
        } catch(LogicException $e) {

        }
        return $this->json($counter, 200);
    }

    #[Route('/{id}', name: 'app_counter_id', methods: ['GET'])]
    public function getCounter(?Counter $counter = null): JsonResponse
    {
        if (!$counter) {
            $counter = [
                'error' => [
                    'code' => 404,
                    'message' => 'Le compteur n\'existe pas'
                ]
                ];
            $code = 404;
        }

        return $this->json($counter, $code ?? 200);
    }

    #[Route('/clear/{id}', name: 'app_counter_clear')]
    public function clear(Counter $counter): JsonResponse
    {
        $this->countdownStateMachine->apply($counter, Counter::TRANSITION_TO_READY);

        return $this->json($counter, 200);
    }
}
